#include "AMI.h"

#include "AppState.h"
#include "SDL_render.h"
#include "SDL_ttf.h"
#include "defines.h"
#include "util.h"
#include "widgets.h"

#include <memory>

constexpr const char* CHINAT_FONT_PATH = "resources/CHINAT.ttf";
constexpr int CHINAT_MEDIUM_PTS = 25;
constexpr int CHINAT_LARGE_PTS = 40;
constexpr int HEADER_Y = 10;

AMI::AMI(SDL_Renderer* renderer)
    : View{renderer}, font_medium{util::load_font(CHINAT_FONT_PATH,
                                                  CHINAT_MEDIUM_PTS)},
      font_large{util::load_font(CHINAT_FONT_PATH, CHINAT_LARGE_PTS)} {
  header =
      std::make_unique<TextWidget>(renderer, font_medium, "Current mission:");
  header->set_position(SCREEN_WIDTH / 2, HEADER_Y);
  header->set_alignment(Alignment::CENTER, Alignment::TOP);

  missions.emplace(Mission::NONE,
                   std::make_unique<TextWidget>(renderer, font_large,
                                                "NO MISSION SELECTED"));
  missions.emplace(
      Mission::ACCELERATION,
      std::make_unique<TextWidget>(renderer, font_large, "ACCELERATION"));
  missions.emplace(Mission::SKIDPAD, std::make_unique<TextWidget>(
                                         renderer, font_large, "SKIDPAD"));
  missions.emplace(Mission::AUTOCROSS, std::make_unique<TextWidget>(
                                           renderer, font_large, "AUTOCROSS"));
  missions.emplace(
      Mission::TRACKDRIVE,
      std::make_unique<TextWidget>(renderer, font_large, "TRACKDRIVE"));
  missions.emplace(Mission::EBS_TEST, std::make_unique<TextWidget>(
                                          renderer, font_large, "EBS TEST"));
  missions.emplace(
      Mission::INSPECTION,
      std::make_unique<TextWidget>(renderer, font_large, "INSPECTION"));
  missions.emplace(
      Mission::MANUAL,
      std::make_unique<TextWidget>(renderer, font_large, "MANUAL DRIVING"));
  for (auto& [mission, widget] : missions) {
    widget->set_position(SCREEN_WIDTH / 2, SCREEN_HEIGHT / 2);
    widget->set_alignment(Alignment::CENTER, Alignment::CENTER);
  }
}

AMI::~AMI() {
  TTF_CloseFont(font_medium);
  TTF_CloseFont(font_large);
}

void AMI::draw(const AppState& state) {
  SDL_SetRenderDrawColor(renderer, 0x00, 0x00, 0x00, 0xFF);
  SDL_RenderClear(renderer);

  header->draw();
  missions[state.get_mission()]->draw();
}