#include "MissionSelect.h"

#include "AppState.h"
#include "DriverView.h"
#include "defines.h"
#include "events.h"
#include "util.h"
#include "widgets.h"

#include <fmt/format.h>

#include <array>
#include <iostream>
#include <memory>

constexpr const char* FT_LOGO_PATH = "resources/Fasttube_Logo-white.bmp";
constexpr const char* AVENIR_FONT_PATH = "resources/Avenir-Book.ttf";
constexpr int AVENIR_PTS = 20;
constexpr const char* CHINAT_FONT_PATH = "resources/CHINAT.ttf";
constexpr int CHINAT_PTS = 24;
constexpr int FT_LOGO_HEIGHT = 40;
constexpr int GAP = 5;
constexpr int CHOOSE_Y = FT_LOGO_HEIGHT + GAP;

constexpr std::array MISSIONS = {"ACCELERATION",  "SKIDPAD",  "AUTOCROSS",
                                 "TRACKDRIVE",    "EBS TEST", "INSPECTION",
                                 "MANUAL DRIVING"};

MissionSelect::MissionSelect(SDL_Renderer* renderer)
    : View{renderer}, avenir{util::load_font(AVENIR_FONT_PATH, AVENIR_PTS)},
      chinat{util::load_font(CHINAT_FONT_PATH, CHINAT_PTS)} {
  ft_logo = std::make_unique<ImageWidget>(renderer, FT_LOGO_PATH);
  ft_logo->set_height(FT_LOGO_HEIGHT);
  ft_logo->set_position(SCREEN_WIDTH / 2, 0);
  ft_logo->set_alignment(Alignment::CENTER, Alignment::TOP);
  widgets.push_back(ft_logo.get());

  choose = std::make_unique<TextWidget>(renderer, avenir, "Choose a mission:");
  choose->set_position(SCREEN_WIDTH / 2, CHOOSE_Y);
  choose->set_alignment(Alignment::CENTER, Alignment::TOP);
  widgets.push_back(choose.get());

  for (const auto mission : MISSIONS) {
    missions.push_back(std::make_unique<TextWidget>(renderer, chinat, mission));
  }
  int choose_height = choose->get_height();
  int missions_y = CHOOSE_Y + choose_height + GAP;
  int missions_height = SCREEN_HEIGHT - CHOOSE_Y - choose_height - 1;
  missions_widget = std::make_unique<ListWidget>(
      renderer, missions[0]->get_height(), Alignment::CENTER);
  missions_widget->set_position(0, missions_y);
  missions_widget->set_width(SCREEN_WIDTH, false);
  missions_widget->set_height(missions_height, false);
  for (const auto& mission : missions) {
    missions_widget->add_element(mission.get());
  }
  widgets.push_back(missions_widget.get());
}

MissionSelect::~MissionSelect() {
  TTF_CloseFont(avenir);
  TTF_CloseFont(chinat);
}

void MissionSelect::draw(const AppState& state) {
  size_t n = static_cast<int>(state.get_mission()) - 1;
  missions_widget->select(n);

  SDL_SetRenderDrawColor(renderer, 0x00, 0x00, 0x00, 0xFF);
  SDL_RenderClear(renderer);
  for (const auto& widget : widgets) {
    widget->draw();
  }
}
