#include "util.h"

#include <SDL2/SDL_image.h>
#include <fmt/format.h>

#include <stdexcept>

namespace util {

SDL_Texture* load_img(SDL_Renderer* renderer, const std::string& path) {
  SDL_Surface* loaded = IMG_Load(path.c_str());
  if (loaded == nullptr) {
    throw std::runtime_error(
        fmt::format("Unable to load image {}: {}", path, IMG_GetError()));
  }
  SDL_Texture* texture = SDL_CreateTextureFromSurface(renderer, loaded);
  if (texture == nullptr) {
    throw std::runtime_error(fmt::format(
        "Unable to convert image {} to texture: {}", path, IMG_GetError()));
  }
  SDL_FreeSurface(loaded);
  return texture;
}

TTF_Font* load_font(const std::string& path, int pt) {
  TTF_Font* font = TTF_OpenFont(path.c_str(), pt);
  if (font == nullptr) {
    throw std::runtime_error(
        fmt::format("Unable to load font {}: {}", path, TTF_GetError()));
  }
  return font;
}

} // namespace util
